Package: emmeans
Type: Package
Title: Estimated Marginal Means, aka Least-Squares Means
Version: 2.0.3
Date: 2026-03-30
Authors@R: c(person("Russell V.", "Lenth", role = c("aut", "cph"), 
                    email = "russell-lenth@uiowa.edu"),
    person("Julia", "Piaskowski", role = c("cre", "aut"),
           email = "julia.piask@gmail.com"),
    person("Balazs", "Banfai", role = "ctb"),
    person("Ben", "Bolker", role = "ctb"),
    person("Paul", "Buerkner", role = "ctb"),
    person("Iago", "Giné-Vázquez", role = "ctb"),
    person("Maxime", "Hervé", role = "ctb"),
    person("Maarten", "Jung", role = "ctb"),
    person("Jonathon", "Love", role = "ctb"),
    person("Fernando", "Miguez", role = "ctb"),
    person("Hannes", "Riebl", role = "ctb"),
    person("Henrik", "Singmann", role = "ctb"))
Maintainer: Julia Piaskowski <julia.piask@gmail.com>
Depends: R (>= 4.1.0)
Imports: estimability (>= 1.4.1), graphics, methods, mvtnorm, numDeriv,
        rlang, stats, utils
Suggests: bayesplot, bayestestR, biglm, brms, car, coda (>= 0.17),
        compositions, ggplot2 (>= 4.0.0), knitr, lattice, lme4,
        lmerTest (>= 2.0.32), logspline, MASS, mediation, mgcv,
        multcomp, multcompView, MuMIn, nlme, ordinal (>= 2014.11-12),
        pbkrtest (>= 0.4-1), rmarkdown, robmixglm, rsm, sandwich,
        scales, splines, testthat, tibble, xtable (>= 1.8-2)
Enhances: CARBayes, coxme, gee, geepack, MCMCglmm, MCMCpack, mice,
        nnet, pscl, rstanarm, sommer, survival
URL: https://rvlenth.github.io/emmeans/,
        https://github.com/rvlenth/emmeans/
BugReports: https://github.com/rvlenth/emmeans/issues
LazyData: yes
ByteCompile: yes
Description: Obtain estimated marginal means (EMMs) for many linear, generalized 
  linear, and mixed models. Compute contrasts or linear functions of EMMs,
  trends, and comparisons of slopes. Plots and other displays.
  Least-squares means are discussed, and the term "estimated marginal means"
  is suggested, in Searle, Speed, and Milliken (1980) Population marginal means 
  in the linear model: An alternative to least squares means, The American 
  Statistician 34(4), 216-221 <doi:10.1080/00031305.1980.10483031>.
License: GPL-2 | GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-08 22:31:47 UTC; jpiaskowski
Author: Russell V. Lenth [aut, cph],
  Julia Piaskowski [cre, aut],
  Balazs Banfai [ctb],
  Ben Bolker [ctb],
  Paul Buerkner [ctb],
  Iago Giné-Vázquez [ctb],
  Maxime Hervé [ctb],
  Maarten Jung [ctb],
  Jonathon Love [ctb],
  Fernando Miguez [ctb],
  Hannes Riebl [ctb],
  Henrik Singmann [ctb]
Repository: CRAN
Date/Publication: 2026-04-09 10:00:20 UTC
Built: R 4.6.0; ; 2026-04-25 16:13:50 UTC; unix
